rm(list=ls())
graphics.off()

library(bbmle)
library(ggplot2)

ctmindata<-read.csv("ctmindata.csv")

#Replace the name of the species accordingly for each analysis
terblan<-subset(ctmindata, ctmindata$Species=="Xenylla humicola")

attach(terblan)

########################## 1. GROUP OF 2 CONSTANT MODELS ####$$$$$$$$$$$$$$$$$$$$$$$$

##Log likelihood function to fit both CONSTANT and CONSTANT THRESHOLD
threshconstmort <- function(m,ct,ftime,ramp,starttemp){
  T0 <- pmax(starttemp,ct)
  tthresh <- pmax(0,(starttemp-ct)/ramp)
  ll <- ifelse(ftime<=tthresh,log(0),log(m)-m*(ftime-tthresh))
  -sum(ll)
}

##CONSTANT FIT (no threshold, ct=StartTemp)
constfit <- 
  bbmle::mle2(threshconstmort,start=list(m=.001),
              data=list(ct=min(StartTemp),ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Brent",
              control=list(maxit=10000),skip.hessian=TRUE,lower=0,upper=100)
constfit

confint(profile(constfit,std.err=.01))

#Extract coefficients for plotting
m <- coef(constfit)[1]

#Plotting the function
quartz()
plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Constant")
temperature <- seq(from=-8,to=27,0.1)
failurerate <- rep(m,length(temperature))
lines(temperature,failurerate, col="green")

#Computing time to failure
pdf.cst <- function(t,a){
  f <-  a*exp(-a*t)
  f*t
}

#time integration 
tc <- numeric()
for(i in 1:nrow(terblan))
  tc[i] <- integrate(pdf.cst,0,Inf,a=m)$value

CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.cst <- cor(CTmin,CTmin.est)^2
var.cst

##plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Constant")



##CONSTANT FIT with THRESHOLD (ct varies, a free parameter)
threshconstfit <- 
  bbmle::mle2(threshconstmort,start=list(m=0.001,ct=min(StartTemp)),
              data=list(ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Nelder-Mead",
              control=list(maxit=10000),skip.hessian=TRUE)
threshconstfit

confint(profile(threshconstfit,std.err=c(.01,.5)))

#Extract coefficients for plotting

m <- coef(threshconstfit)["m"]
ct <- coef(threshconstfit)["ct"]
ct

ctthrsh <- function(m,ct,x){
  ifelse(x>ct,0,m)
}

#Plotting the function
quartz()
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Constant Threshold")
plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.2), pch=16,main="Constant Threshold")
failurerate <- ctthrsh(m,ct,temperature)
lines(temperature,failurerate, col="green")

#Computing time to failure
pdf.cst.th <- function(m,ct,t){
  ifelse(t>ct, f <-  0, f <-  m*exp(-m*t))
  f*t
}

##time integration 
tc <- numeric()
for(i in 1:nrow(terblan))
  tc[i] <- integrate(pdf.cst.th,0,Inf,m=m,ct=ct)$value

CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.cst.th <- cor(CTmin,CTmin.est)^2
var.cst.th

##plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Constant Threshold")







##################  2. GROUP OF 2 EXPONENTIAL MODELS ####################

##Log likelihood function to fit both the EXPONENTIAL and EXPONENTIAL THRESHOLD functions
threshexpmort <- function(m0,b,ct,ftime,ramp,starttemp){
  T0 <- pmax(starttemp,ct)
  tthresh <- pmax(0,(starttemp-ct)/ramp)
  A <- exp(m0+b*(ct-starttemp))
  ll <- ifelse(ftime<=tthresh,log(0),ifelse(ramp==0,log(A)+A*ftime,
                                            log(A)+(b*ramp*ftime)-
                                              (A/b/ramp*(exp(b*ramp*ftime)-exp(b*ramp*tthresh)))))
  -sum(ll)
}


expfit <- 
  bbmle::mle2(threshexpmort,start=list(m0=-10,b=0.001),
              data=list(ct=min(StartTemp),ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Nelder-Mead",
              control=list(maxit=10000),skip.hessian=TRUE)
expfit

##Extract coefficients for plotting
m <- coef(expfit)["m0"]
b <- coef(expfit)["b"]

expf <- function(m0,b,x){
  exp(m0)*exp(b*(min(StartTemp)-x))
}


#Plotting the function
quartz()
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Exponential")
plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.07), pch=16,main="Exponential")
failurerate <- expf(m,b,temperature)
lines(temperature,failurerate, col="green")


##Computing time to failure
pdf.exp <- function(x,m0,b,ct,starttemp,ramp){
  #This is the PDF for the threshold exponential model
  T0 <- pmax(starttemp,ct)
  tthresh <- pmax(0,(starttemp-ct)/ramp)
  A <- exp(m0+b*(ct-starttemp))
  pdf <- x*ifelse(x<=tthresh,0,A*exp(b*ramp*x)*exp(-(A/b/ramp*(exp(b*ramp*x)-exp(b*ramp*tthresh)))))
  pdf
}


##########time integration
tc <- numeric()
for(i in 1:nrow(terblan)){
  tc.temp <- rep(0,101)
  for(j in 1:100){
    try({tc.temp[j] <- integrate(pdf.exp,10*(j-1),10*(j),m0=m,b=b,ct=min(StartTemp),starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  }
  try({tc.temp[101] <- integrate(pdf.exp,1000,Inf,m0=m,b=b,ct=min(StartTemp),starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  tc[i] <- sum(tc.temp)
}

 
CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.exp <- cor(CTmin,CTmin.est)^2
var.exp

##plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Exponential")

#Calculate CIs
eprof <- bbmle::profile(expfit,std.err=c(.1,.01),which=c("m0","b"))
confint(eprof)






##EXPONENTIAL THRESHOLD, ct is a free parameter
threshexpfit <- 
  bbmle::mle2(threshexpmort,start=list(m0=coef(expfit)[1],b=coef(expfit)[2],ct=min(StartTemp)),
              data=list(ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Nelder-Mead",
              control=list(maxit=10000),skip.hessian=TRUE)

threshexpfit

##Extract coefficients for plotting
m <- coef(threshexpfit)["m0"]
b <- coef(threshexpfit)["b"]
ct <- coef(threshexpfit)["ct"]
ct

#changed sign here >
expf.th <- function(m0,b,ct,x){
  ifelse(x>=ct,exp(m0),exp(m0)*exp(b*(ct-x)))
}

#Plotting the function
quartz()
plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Exponential Threshold")
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.07), pch=16,main="Exponential Threshold")
failurerate <- expf.th(m,b,ct,temperature)
lines(temperature,failurerate, col="green")


#Computing time to failure
##it is the same function as above
# pdf.exp <- function(x,m0,b,ct,starttemp,ramp){
#   #This is the PDF for the threshold exponential model
#   T0 <- pmax(starttemp,ct)
#   tthresh <- pmax(0,(ct-starttemp)/ramp)
#   A <- exp(m0+b*(starttemp-ct))
#   pdf <- x*ifelse(x<=tthresh,0,A*exp(b*ramp*x)*exp(-(A/b/ramp*(exp(b*ramp*x)-exp(b*ramp*tthresh)))))
#   pdf
# }

##time integration 
tc <- numeric()
for(i in 1:nrow(terblan)){
  tc.temp <- rep(0,101)
  for(j in 1:100){
    try({tc.temp[j] <- integrate(pdf.exp,10*(j-1),10*(j),m0=m,b=b,ct=ct,starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  }
  try({tc.temp[101] <- integrate(pdf.exp,1000,Inf,m0=m,b=b,ct=ct,starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  tc[i] <- sum(tc.temp)
}


CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.exp.th <- cor(CTmin,CTmin.est)^2
var.exp.th

#plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Exponential Threshold")

#Calculate CIs
eprof <- bbmle::profile(threshexpfit,std.err=c(.1,.01),which=c("m0","b","ct"))
confint(eprof)









#########  3. GROUP OF 2 ZERO EXPONENTIAL MODELS ##########################

##LL function to fit both the ZERO EXPONENTIAL and ZERO EXPONENTIAL THRESHOLD functions
##mortality rate as a function continuous at the threshold
zerothreshexpmort <- function(m0,b,ct,ftime,ramp,starttemp){
  T0 <- pmax(starttemp,ct)
  tthresh <- pmax(0,(starttemp-ct)/ramp)
  A <- exp(m0+b*(ct-starttemp))
  ll <- ifelse(ftime<=tthresh,0,ifelse(ramp==0,log(A)+(A-exp(m0))*ftime,(A*exp(b*ramp*ftime)-exp(m0))*exp(-(A/b/ramp*(exp(b*ramp*ftime)-exp(b*ramp*tthresh))-exp(m0)*(ftime-tthresh)))))
  -sum(log(ll))
}


##ZERO EXPONENTIAL, ct=StartTemp
zeroexpfit <- 
  bbmle::mle2(zerothreshexpmort,start=list(m0=-7,b=0.001),
              data=list(ct=min(StartTemp),ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Nelder-Mead",
              control=list(maxit=10000),skip.hessian=TRUE)
zeroexpfit

#Extract coefficients for plotting
m <- coef(zeroexpfit)["m0"]
b <- coef(zeroexpfit)["b"]


####Change ct to start temp 
zexpf <- function(m0,b,x){
  exp(m0)*exp(b*(min(StartTemp)-x))-exp(m0)
}

quartz()
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Zero Exponential")
plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.07), pch=16,main="Zero Exponential")
failurerate <- zexpf(m,b,temperature)
lines(temperature,failurerate, col="green")


##Computing time to failure
zpdf.exp <- function(x,m0,b,ct,starttemp,ramp){
  #This is the PDF for the threshold exponential model
  T0 <- pmax(starttemp,ct)
  tthresh <- pmax(0,(starttemp-ct)/ramp)
  A <- exp(m0+b*(ct-starttemp))
  pdf <- x*ifelse(x<=tthresh,0,(A*exp(b*ramp*x)-exp(m0))*exp(-(A/b/ramp*(exp(b*ramp*x)-exp(b*ramp*tthresh))-exp(m0)*(x-tthresh))))
  pdf
}


##time integration
tc <- numeric()
for(i in 1:nrow(terblan)){
  tc.temp <- rep(0,101)
  for(j in 1:100){
    try({tc.temp[j] <- integrate(zpdf.exp,10*(j-1),10*(j),m0=m,b=b,ct=min(StartTemp),starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  }
  try({tc.temp[101] <- integrate(zpdf.exp,1000,Inf,m0=m,b=b,ct=min(StartTemp),starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  tc[i] <- sum(tc.temp)
}

CTmin.est <- StartTemp-RampRate*tc

#Compute variance
var.zexp <- cor(CTmin,CTmin.est)^2
var.zexp

##plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Zero Exponential")

#Calculate CIs
eprof <- bbmle::profile(zeroexpfit,std.err=c(.1,.01),which=c("m0","b"))
confint(eprof)




#ZERO EXPONENTIAL THRESHOLD, ct varies freely
zerothreshexpfit<-
  bbmle::mle2(zerothreshexpmort,start=list(m0=-10,b=0.001,ct=min(StartTemp)),
              data=list(ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Nelder-Mead",
              control=list(maxit=10000),skip.hessian=TRUE)
zerothreshexpfit


##Extract coefficients for plotting
m <- coef(zerothreshexpfit)["m0"]
b <- coef(zerothreshexpfit)["b"]
ct <- coef(zerothreshexpfit)["ct"]
ct

zexpf.th <- function(m0,b,ct,x){
  ifelse(x>=ct,0,exp(m0)*exp(b*(ct-x))-exp(m0))
}


quartz()
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Zero Exponential Threshold")
plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.07), pch=16,main="Zero Exponential Threshold")
failurerate <- zexpf.th(m,b,ct,temperature)
lines(temperature,failurerate, col="green")

###time integration 
tc <- numeric()
for(i in 1:nrow(terblan)){
  tc.temp <- rep(0,101)
  for(j in 1:100){
    try({tc.temp[j] <- integrate(zpdf.exp,10*(j-1),10*(j),m0=m,b=b,ct=ct,starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  }
  try({tc.temp[101] <- integrate(zpdf.exp,1000,Inf,m0=m,b=b,ct=ct,starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  tc[i] <- sum(tc.temp)
}

CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.zexp.th <- cor(CTmin,CTmin.est)^2
var.zexp.th

####plot CTmax estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Zero Exponential Threshold")

#Calculate CIs
eprof <- bbmle::profile(zerothreshexpfit,std.err=c(.1,.01),which=c("m0","b","ct"))
confint(eprof)









############# 4. GROUP of 2 ZERO POWER MODELS ##################

####POWER FUNCTIONS
zerothreshpowermort <- function(a,b,ct,ftime,ramp,starttemp){
  a <- exp(a)
  tc <- pmax(0,ramp*ftime-pmin(starttemp-ct,0))
  ll <- ifelse(ramp==0,(a)*(ct-startemp)^b*exp(a*(ct-starttemp)^b*tc),
               (a*(tc)^b)*exp(-(a/ramp/(b+1)*(tc^(b+1)))))
  -sum(log(ll))
}


##ZERO POWER MODEL, fixed ct=StartTemp
powfit <- mle2(zerothreshpowermort,start=list(a=-10,b=0.001),
               data=list(ftime=terblan$FailTime,ramp=terblan$RampRate,
                         starttemp=terblan$StartTemp,ct=min(StartTemp)),method="Nelder-Mead",
               control=list(maxit=10000))
powfit


##Extract coefficients for plotting
a <- exp(coef(powfit)["a"])
b <- coef(powfit)["b"]

powf <- function(a,b,x){
  a*(min(StartTemp)-x)^b
}


quartz()
plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Zero Power")
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.07), pch=16,main="Zero Power")
failurerate <- powf(a,b,temperature)
lines(temperature,failurerate, col="green")


##Computing time to failure
pdf.pow <- function(x,a,b,ct,starttemp,ramp){  
  #This is the PDF for the threshold power model
  tc <- pmax(0,ramp*x-pmin(starttemp-ct,0))
  ts <- pmax(0,ct-starttemp)
  pdf <- x*(a*(tc)^b)*exp(-(a/ramp/(b+1)*(tc^(b+1)-ts^(b+1)))) 
  pdf
}


##time integration
tc <- numeric()
for(i in 1:nrow(terblan)){
  tc.temp <- rep(0,101)
  for(j in 1:100){
    try({tc.temp[j] <- integrate(pdf.pow,10*(j-1),10*(j),a=a,b=b,ct=min(StartTemp),starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  }
  try({tc.temp[101] <- integrate(pdf.pow,1000,Inf,a=a,b=b,ct=min(StartTemp),starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  tc[i] <- sum(tc.temp)
}


CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.pow <- cor(CTmin,CTmin.est)^2
var.pow

##plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Zero Power")

#Calculate CIs
eprof <- bbmle::profile(powfit,std.err=c(.1,.01),which=c("a","b"))
confint(eprof)




##ZERO POWER THRESHOLD MODEL, ct is a free parameter
zerothreshpowfit <- 
  bbmle::mle2(zerothreshpowermort,start=list(a=-10,b=0.001,ct=min(StartTemp)),
              data=list(ftime=terblan$FailTime,ramp=terblan$RampRate,
                        starttemp=terblan$StartTemp),method="Nelder-Mead",
              control=list(maxit=10000))

zerothreshpowfit


##Extract coefficients for plotting
a <- exp(coef(zerothreshpowfit)["a"])
b <- coef(zerothreshpowfit)["b"]
ct <- coef(zerothreshpowfit)["ct"]
ct

pow.thf <- function(a,b,ct,x){
  ifelse(x>=ct,0,a*(ct-x)^b)
}

quartz()
plot(CTmin,M, xlim=c(-8,27), ylim=c(0.000001,1), pch=16, log = "y",main="Zero Power Threshold")
#plot(CTmin,M, xlim=c(-8,27), ylim=c(0,0.07), pch=16,main="Zero Power Threshold")
failurerate <- pow.thf(a,b,ct,temperature)
lines(temperature,failurerate, col="green")


#######Computing time to failure
tc <- numeric()
for(i in 1:nrow(terblan)){
  tc.temp <- rep(0,101)
  for(j in 1:100){
    try({tc.temp[j] <- integrate(pdf.pow,10*(j-1),10*(j),a=a,b=b,ct=ct,starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  }
  try({tc.temp[101] <- integrate(pdf.pow,1000,Inf,a=a,b=b,ct=ct,starttemp=terblan$StartTemp[i],ramp=terblan$RampRate[i])$value},silent=TRUE)
  tc[i] <- sum(tc.temp)
}

CTmin.est <- StartTemp-RampRate*tc

##Compute variance
var.pow.th <- cor(CTmin,CTmin.est)^2
var.pow.th

##plot CTmin estimates
quartz()
plot(CTmin,CTmin.est,pch=16, main="CTmin.obs vs. CTmin.est - Zero Power Threshold")

#Calculate CIs
eprof <- bbmle::profile(zerothreshpowfit,std.err=c(.1,.01),which=c("a","b","ct"))
confint(eprof)




###################Summary Table
modsummary <- 
  data.frame(AIC(zerothreshexpfit,zeroexpfit,zerothreshpowfit,powfit,
                 threshexpfit,expfit,constfit,threshconstfit,powfit),
             loglik=sapply(list(zerothreshexpfit,zeroexpfit,zerothreshpowfit,
                                powfit,threshexpfit,expfit,constfit,
                                threshconstfit,powfit),logLik))
rownames(modsummary) <- 
  c("Zero mortality exponential threshold","Zero mortality exponential",
    "Zero mortality power threshold","Zero mortality power fit",
    "Exponential threshold","Exponential","Constant mortality",
    "Threshold constant mortality","Power")
modsummary["Power",]$df <- modsummary["Power",]$df+1
modsummary["Power",]$AIC <- modsummary["Power",]$AIC+2
modsummary[order(modsummary$AIC),]

############ AICc ##################
#constant
AICc(constfit)
#constant threshold
AICc(threshconstfit)
#exponential
AICc(expfit)
#exponential threshold
AICc(threshexpfit)
#zero exponential
AICc(zeroexpfit)
#zero exponential threshold
AICc(zerothreshexpfit)
#zero power
AICc(powfit)
#zero power threshold
AICc(zerothreshpowfit)

#### Variance explained by the statistical effects #################
#constant
var.cst
#constant threshold
var.cst.th
#exponential
var.exp
#exponential threshold
var.exp.th
#zero exponential
var.zexp
#zero exponential threshold
var.zexp.th
#zero power
var.pow
#zero power threshold
var.pow.th


